from unittest import TestCase
from kfk.kubectl_command_builder import Kubectl
from kfk.config import KUBECTL_PATH


class TestKubectl(TestCase):
    def test_get(self):
        self.assertEqual(Kubectl().get().build(), "{kubectl} get".format(kubectl=KUBECTL_PATH))

    def test_create(self):
        self.assertEqual(Kubectl().create().build(), "{kubectl} create".format(kubectl=KUBECTL_PATH))

    def test_apply(self):
        self.assertEqual(Kubectl().apply().build(), "{kubectl} apply".format(kubectl=KUBECTL_PATH))

    def test_describe(self):
        self.assertEqual(Kubectl().describe().build(), "{kubectl} describe".format(kubectl=KUBECTL_PATH))

    def test_delete(self):
        self.assertEqual(Kubectl().delete().build(), "{kubectl} delete".format(kubectl=KUBECTL_PATH))

    def test_exec(self):
        self.assertEqual(
            Kubectl().exec("-it", "test-pod").container("test-container").namespace("test-namespace").exec_command(
                "echo 'test'").build(),
            "{kubectl} exec -it test-pod -c test-container -n test-namespace -- bash -c \"echo 'test'\"".format(
                kubectl=KUBECTL_PATH))

    def test_kafkas_all(self):
        self.assertEqual(Kubectl().get().kafkas().build(), "{kubectl} get kafkas".format(kubectl=KUBECTL_PATH))

    def test_kafkas_specific(self):
        self.assertEqual(Kubectl().get().kafkas("my-cluster").build(),
                         "{kubectl} get kafkas my-cluster".format(kubectl=KUBECTL_PATH))

    def test_kafkatopics_all(self):
        self.assertEqual(Kubectl().get().kafkatopics().build(),
                         "{kubectl} get kafkatopics".format(kubectl=KUBECTL_PATH))

    def test_kafkatopics_specific(self):
        self.assertEqual(Kubectl().get().kafkatopics("my-topic").build(),
                         "{kubectl} get kafkatopics my-topic".format(kubectl=KUBECTL_PATH))

    def test_kafkausers_all(self):
        self.assertEqual(Kubectl().get().kafkausers().build(), "{kubectl} get kafkausers".format(kubectl=KUBECTL_PATH))

    def test_kafkausers_specific(self):
        self.assertEqual(Kubectl().get().kafkausers("my-user").build(),
                         "{kubectl} get kafkausers my-user".format(kubectl=KUBECTL_PATH))

    def test_configmap_yaml(self):
        self.assertEqual(Kubectl().get().configmap("my-cluster-kafka-config").output("yaml").build(),
                         "{kubectl} get configmap my-cluster-kafka-config -o yaml".format(kubectl=KUBECTL_PATH))

    def test_label(self):
        self.assertEqual(Kubectl().get().kafkatopics().label("app=test").build(),
                         "{kubectl} get kafkatopics -l app=test".format(kubectl=KUBECTL_PATH))

    def test_labels(self):
        self.assertEqual(Kubectl().get().kafkatopics().label("app=test, included=true").build(),
                         "{kubectl} get kafkatopics -l app=test, included=true".format(kubectl=KUBECTL_PATH))

    def test_namespace(self):
        self.assertEqual(Kubectl().get().kafkatopics().namespace("test").build(),
                         "{kubectl} get kafkatopics -n test".format(kubectl=KUBECTL_PATH))

    def test_namespace_all(self):
        self.assertEqual(Kubectl().get().kafkatopics().namespace().build(),
                         "{kubectl} get kafkatopics --all-namespaces".format(kubectl=KUBECTL_PATH))

    def test_label_namespace(self):
        self.assertEqual(Kubectl().get().kafkatopics().label("app=test").namespace("test").build(),
                         "{kubectl} get kafkatopics -l app=test -n test".format(kubectl=KUBECTL_PATH))

    def test_cluster(self):
        self.assertEqual(Kubectl().get().kafkas().container("test-container").build(),
                         "{kubectl} get kafkas -c test-container".format(kubectl=KUBECTL_PATH))

    def test_output(self):
        self.assertEqual(Kubectl().get().kafkas().output("yaml").build(),
                         "{kubectl} get kafkas -o yaml".format(kubectl=KUBECTL_PATH))

    def test_create_from_file(self):
        self.assertEqual(Kubectl().create().from_file("/a/test/path/testfile.yaml").build(),
                         "{kubectl} create -f /a/test/path/testfile.yaml".format(kubectl=KUBECTL_PATH))

    def test_create_secret(self):
        self.assertEqual(Kubectl().create().secret("docker-registry", "secret-name", "--docker-username=tiger",
                                                   "--docker-password=pass113").build(),
                         "{kubectl} create secret docker-registry secret-name --docker-username=tiger "
                         "--docker-password=pass113".format(kubectl=KUBECTL_PATH))



from unittest import TestCase, mock
from click.testing import CliRunner
from kfk.commands.users import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkUsers(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.user = "my-user"

    def test_no_option(self):
        result = self.runner.invoke(kfk, ['users', '--user', self.user, '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        assert "Missing options: kfk users" in result.output

    @mock.patch('kfk.commands.users.os')
    def test_list_users(self, mock_os):
        result = self.runner.invoke(kfk, ['users', '--list', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkausers().label("strimzi.io/cluster={cluster}").namespace(self.namespace).build().format(
                cluster=self.cluster))

    @mock.patch('kfk.commands.users.os')
    def test_describe_user(self, mock_os):
        result = self.runner.invoke(kfk, ['users', '--describe', '--user', self.user, '-c', self.cluster, '-n',
                                          self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().describe().kafkausers(self.user).namespace(self.namespace).build())

    @mock.patch('kfk.commands.users.os')
    def test_describe_user_output_yaml(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['users', '--describe', '--user', self.user, '-c', self.cluster, '-n',
                                     self.namespace, '-o',
                                     'yaml'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkausers(self.user).namespace(self.namespace).output("yaml").build())

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commands.users.os')
    def test_create_user(self, mock_os, mock_create_temp_file):
        result = self.runner.invoke(kfk,
                                    ['users', '--create', '--user', self.user, '--authentication-type', 'tls', '-c',
                                     self.cluster, '-n', self.namespace])

        assert result.exit_code == 0

        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            expected_user_yaml = file.read()
            result_user_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_user_yaml == result_user_yaml

    def test_create_user_without_auth(self):
        result = self.runner.invoke(kfk,
                                    ['users', '--create', '--user', self.user, '-c', self.cluster, '-n',
                                     self.namespace])
        assert result.exit_code == 2

    def test_create_user_with_wrong_authentication_type(self):
        result = self.runner.invoke(kfk,
                                    ['users', '--create', '--user', self.user, '--authentication-type', 'auth',
                                     '--authorization-type', 'simple', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 2

    def test_create_user_with_wrong_authorization_type(self):
        result = self.runner.invoke(kfk,
                                    ['users', '--create', '--user', self.user, '--authentication-type', 'tls',
                                     '--authorization-type', 'auth', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 2

    @mock.patch('kfk.commands.users.os')
    def test_delete_user(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['users', '--delete', '--user', self.user, '-c', self.cluster, '-n',
                                     self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().delete().kafkausers(self.user).namespace(self.namespace).build())

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_with_no_params(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            expected_user_yaml = file.read()
            mock_get_resource_yaml.return_value = expected_user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '-c', self.cluster, '-n',
                                         self.namespace])
            assert result.exit_code == 0

            result_user_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_authentication(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authentication-type',
                                         'scram-sha-512', '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authentication_scram.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_authorization(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authorization-type',
                                         'simple', '--add-acl', '--operation', 'Read', '--resource-type', 'topic',
                                         '--resource-name', 'my-topic', '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_authorization_none(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authorization-type',
                                         'none', '--add-acl', '--operation', 'Read', '--resource-type', 'topic',
                                         '--resource-name', 'my-topic', '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_authorization_with_two_acl_operation(self, mock_os, mock_get_resource_yaml,
                                                                 mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authorization-type',
                                         'simple', '--add-acl', '--operation', 'Read', '--operation', 'Write',
                                         '--resource-type', 'topic', '--resource-name', 'my-topic', '-c', self.cluster,
                                         '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_two_topic_acls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_authorization_none_with_two_acl_operation(self, mock_os, mock_get_resource_yaml,
                                                                      mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_two_topic_acls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authorization-type',
                                         'none', '--add-acl', '--operation', 'Read', '--operation', 'Write',
                                         '--resource-type', 'topic', '--resource-name', 'my-topic', '-c', self.cluster,
                                         '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_authorization_delete_acls_for_one_operation(self, mock_os, mock_get_resource_yaml,
                                                                        mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_three_topic_acls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authorization-type',
                                         'simple', '--delete-acl', '--operation', 'Describe',
                                         '--resource-type', 'topic', '--resource-name', 'my-topic', '-c', self.cluster,
                                         '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_two_topic_acls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_authorization_delete_acls_for_two_operation(self, mock_os, mock_get_resource_yaml,
                                                                        mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_three_topic_acls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authorization-type',
                                         'simple', '--delete-acl', '--operation', 'Describe', '--operation', 'Write',
                                         '--resource-type', 'topic', '--resource-name', 'my-topic', '-c', self.cluster,
                                         '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]

                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_quota(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authentication-type',
                                         'scram-sha-512', '--quota', 'request_percentage=55',
                                         '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_one_quota.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_for_two_quotas(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--authentication-type',
                                         'scram-sha-512', '--quota', 'request_percentage=55',
                                         '--quota', 'consumer_byte_rate=2097152',
                                         '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_two_quotas.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_with_two_quotas_delete_one_quota(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_two_quotas.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--delete-quota',
                                         'consumer_byte_rate',
                                         '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_one_quota.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_alter_user_with_two_quotas_delete_two_quotas(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_two_quotas.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['users', '--alter', '--user', self.user, '--delete-quota',
                                         'consumerByteRate',
                                         '--delete-quota', 'requestPercentage', '-c', self.cluster, '-n',
                                         self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_quotas_empty.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml



from unittest import TestCase, mock
from click.testing import CliRunner
from kfk.commands.operator import kfk
from kfk.kubectl_command_builder import Kubectl
from kfk.config import STRIMZI_PATH


class TestKfkOperator(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.installation_file_count = 28

    @mock.patch('kfk.commands.operator.os.system')
    def test_install_strimzi(self, mock_os_system):
        result = self.runner.invoke(kfk, ['operator', '--install', '-n', self.namespace])
        assert result.exit_code == 0
        assert mock_os_system.call_count == self.installation_file_count



from unittest import TestCase, mock
from click.testing import CliRunner
from kfk.commands.console import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkConsole(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"

    @mock.patch('kfk.commands.console.os')
    def test_console_consumer(self, mock_os):
        result = self.runner.invoke(kfk, ['console-consumer', '--topic', self.topic, '-c', self.cluster, '-n',
                                          self.namespace])
        assert result.exit_code == 0

        native_command = "bin/kafka-console-consumer.sh --bootstrap-server my-cluster-kafka-brokers:9092 --topic {" \
                         "topic} "
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(cluster=self.cluster, topic=self.topic))

    @mock.patch('kfk.commons.transfer_file_to_container')
    @mock.patch('kfk.commands.console.os')
    def test_console_consumer_with_consumer_config(self, mock_os, mock_transfer_file_to_container):
        result = self.runner.invoke(kfk, ['console-consumer', '--topic', self.topic, '--consumer.config',
                                          'files/client.properties', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0

        native_command = "bin/kafka-console-consumer.sh --bootstrap-server {cluster}-kafka-brokers:9093 --topic {" \
                         "topic}  --consumer.config /tmp/client.properties;rm -rf /tmp/truststore.jks;rm -rf " \
                         "/tmp/user.p12;rm -rf /tmp/client.properties;"
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(cluster=self.cluster, topic=self.topic))

    @mock.patch('kfk.commands.console.os')
    def test_console_consumer_with_from_beginning(self, mock_os):
        from_beginning = True
        result = self.runner.invoke(kfk, ['console-consumer', '--topic', self.topic, '-c', self.cluster, '-n',
                                          self.namespace, '--from-beginning'])
        assert result.exit_code == 0

        native_command = "bin/kafka-console-consumer.sh --bootstrap-server my-cluster-kafka-brokers:9092 --topic {" \
                         "topic} {from_beginning}"
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(cluster=self.cluster, topic=self.topic,
                                               from_beginning=(from_beginning and '--from-beginning' or '')))

    @mock.patch('kfk.commands.console.os')
    def test_console_producer(self, mock_os):
        result = self.runner.invoke(kfk, ['console-producer', '--topic', self.topic, '-c', self.cluster, '-n',
                                          self.namespace])
        assert result.exit_code == 0
        native_command = "bin/kafka-console-producer.sh --broker-list my-cluster-kafka-brokers:9092 --topic {topic}"
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(cluster=self.cluster, topic=self.topic))

    @mock.patch('kfk.commons.transfer_file_to_container')
    @mock.patch('kfk.commands.console.os')
    def test_console_producer_with_producer_config(self, mock_os, mock_transfer_file_to_container):
        result = self.runner.invoke(kfk, ['console-producer', '--topic', self.topic, '--producer.config',
                                          'files/client.properties', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        native_command = "bin/kafka-console-producer.sh --broker-list {cluster}-kafka-brokers:9093 --topic {topic} " \
                         "--producer.config /tmp/client.properties;" \
                         "rm -rf /tmp/truststore.jks;rm -rf /tmp/user.p12;rm -rf /tmp/client.properties;"

        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(cluster=self.cluster, topic=self.topic))



from unittest import TestCase, mock

import click
import yaml

from click.testing import CliRunner
from kfk.commands.connect.clusters import connect
from kfk.kubectl_command_builder import Kubectl


class TestKfkConnect(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-connect-cluster"
        self.namespace = "kafka"
        self.connect_config_file = "files/connect.properties"
        self.connector_config_file_1 = "files/twitter-connector.properties"
        self.connector_config_file_2 = "files/file-stream-connector.properties"
        self.registry_userpass = "someuserpass"
        self.registry_server = "quay.io/systemcraftsman/test-connect-cluster:latest"
        self.push_secret_name = f"{self.cluster}-push-secret"

    def test_no_option(self):
        result = self.runner.invoke(connect, ['clusters', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        assert "Missing options: kfk connect" in result.output

    def test_create_custer_without_config_file(self):
        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster, '-n', self.namespace])

        assert result.exit_code == 2

    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor, mock_create_temp_file,
                            mock_click_prompt):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster, self.connect_config_file,
                                     '-n', self.namespace])
        assert result.exit_code == 0

        result_connect_yaml = mock_create_temp_file.call_args[0][0]

        with open(r'files/yaml/kafka-connect.yaml') as file:
            expected_connect_yaml = file.read()

            assert expected_connect_yaml == result_connect_yaml

        mock_os.system.assert_any_call(
            Kubectl().create().secret("docker-registry", self.push_secret_name, "--docker-username={username}",
                                      "--docker-password={password} --docker-server={registry_server}").namespace(
                self.namespace).build().format(username=self.registry_userpass, password=self.registry_userpass,
                                               registry_server=self.registry_server))

    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_with_image(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                       mock_create_temp_file, mock_click_prompt):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster,
                                     'files/connect_with_only_image.properties', '-n', self.namespace])
        assert result.exit_code == 0

        result_connect_yaml = mock_create_temp_file.call_args[0][0]

        with open(r'files/yaml/kafka-connect_with_image.yaml') as file:
            expected_connect_yaml = file.read()

            assert expected_connect_yaml == result_connect_yaml

        try:
            mock_os.system.assert_any_call(
                Kubectl().create().secret("docker-registry", self.push_secret_name, "--docker-username={username}",
                                          "--docker-password={password} --docker-server={registry_server}").namespace(
                    self.namespace).build().format(username=self.registry_userpass, password=self.registry_userpass,
                                                   registry_server=self.registry_server))
        except Exception:
            self.assertRaises(AssertionError)

    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_with_command_error(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                               mock_create_temp_file,
                                               mock_click_prompt):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass
        mock_os.system.return_value = 1

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster, self.connect_config_file,
                                     '-n', self.namespace])
        assert result.exit_code == 0

        assert mock_os.system.call_count == 1

    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_with_zip_jar_plugins(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                                 mock_create_temp_file, mock_click_prompt):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster,
                                     'files/connect_with_zip_jar_plugins.properties', '-n', self.namespace])
        assert result.exit_code == 0

        result_connect_yaml = mock_create_temp_file.call_args[0][0]

        with open(r'files/yaml/kafka-connect_with_zip_jar_plugins.yaml') as file:
            expected_connect_yaml = file.read()

            assert expected_connect_yaml == result_connect_yaml

        mock_os.system.assert_any_call(
            Kubectl().create().secret("docker-registry", self.push_secret_name, "--docker-username={username}",
                                      "--docker-password={password} --docker-server={registry_server}").namespace(
                self.namespace).build().format(
                username=self.registry_userpass, password=self.registry_userpass, registry_server=self.registry_server))

    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_with_invalid_url(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                             mock_create_temp_file, mock_click_prompt):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster,
                                     "files/connect_with_invalid_url.properties",
                                     '-n', self.namespace])
        assert result.exit_code == 1

        self.assertRaises(click.ClickException)

    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_with_yes_flag(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                          mock_create_temp_file, mock_click_prompt):
        mock_click_confirm.return_value = False
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster, self.connect_config_file,
                                     '-n', self.namespace, '-y'])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-connect.yaml') as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_os.system.assert_any_call(
            Kubectl().create().secret("docker-registry", self.push_secret_name, "--docker-username={username}",
                                      "--docker-password={password} --docker-server={registry_server}").namespace(
                self.namespace).build().format(
                username=self.registry_userpass, password=self.registry_userpass, registry_server=self.registry_server))

    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_three_replicas(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                           mock_create_temp_file, mock_click_prompt):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster, '--replicas', 3,
                                     self.connect_config_file, '-n', self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-connect_with_three_replicas.yaml') as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_os.system.assert_any_call(
            Kubectl().create().secret("docker-registry", self.push_secret_name, "--docker-username={username}",
                                      "--docker-password={password} --docker-server={registry_server}").namespace(
                self.namespace).build().format(
                username=self.registry_userpass, password=self.registry_userpass, registry_server=self.registry_server))

    @mock.patch('kfk.commands.connect.connectors.create_temp_file')
    @mock.patch('kfk.commands.connect.connectors.os')
    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_with_connector_config(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                                  mock_create_temp_file, mock_click_prompt, mock_os_connectors,
                                                  mock_create_temp_file_connectors):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass
        mock_os.system.return_value = 0

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster, self.connect_config_file,
                                     self.connector_config_file_1, '-n', self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-connect.yaml') as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_os.system.assert_any_call(
            Kubectl().create().secret("docker-registry", self.push_secret_name, "--docker-username={username}",
                                      "--docker-password={password} --docker-server={registry_server}").namespace(
                self.namespace).build().format(username=self.registry_userpass, password=self.registry_userpass,
                                               registry_server=self.registry_server))

        with open(r'files/yaml/kafka-connect-connector-twitter.yaml') as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file_connectors.call_args[0][0]
            assert expected_connector_yaml == result_connector_yaml

    @mock.patch('kfk.commands.connect.connectors.create_temp_file')
    @mock.patch('kfk.commands.connect.connectors.os')
    @mock.patch('kfk.commands.connect.clusters.click.prompt')
    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commands.connect.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_create_cluster_with_two_connectors_config(self, mock_os, mock_click_confirm,
                                                       mock_open_file_in_system_editor,
                                                       mock_create_temp_file, mock_click_prompt, mock_os_connectors,
                                                       mock_create_temp_file_connectors):
        mock_click_confirm.return_value = True
        mock_click_prompt.return_value = self.registry_userpass
        mock_os.system.return_value = 0

        result = self.runner.invoke(connect,
                                    ['clusters', '--create', '--cluster', self.cluster, self.connect_config_file,
                                     self.connector_config_file_1, self.connector_config_file_2, '-n', self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-connect.yaml') as file:
            expected_connect_yaml = file.read()
            result_connect_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connect_yaml == result_connect_yaml

        mock_os.system.assert_any_call(
            Kubectl().create().secret("docker-registry", self.push_secret_name, "--docker-username={username}",
                                      "--docker-password={password} --docker-server={registry_server}").namespace(
                self.namespace).build().format(username=self.registry_userpass, password=self.registry_userpass,
                                               registry_server=self.registry_server))

        with open(r'files/yaml/kafka-connect-connector-twitter.yaml') as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file_connectors.call_args_list[0][0][0]
            assert expected_connector_yaml == result_connector_yaml

        with open(r'files/yaml/kafka-connect-connector-file-stream.yaml') as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file_connectors.call_args_list[1][0][0]
            assert expected_connector_yaml == result_connector_yaml

    @mock.patch('kfk.commands.connect.clusters.os')
    def test_list_clusters(self, mock_os):
        result = self.runner.invoke(connect, ['clusters', '--list', '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().get().kafkaconnects().namespace(self.namespace).build())

    @mock.patch('kfk.commands.connect.clusters.os')
    def test_list_clusters_all_namespaces(self, mock_os):
        result = self.runner.invoke(connect, ['clusters', '--list'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().get().kafkaconnects().namespace().build())

    @mock.patch('kfk.commands.connect.clusters.os')
    def test_describe_cluster(self, mock_os):
        result = self.runner.invoke(connect,
                                    ['clusters', '--describe', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkaconnects(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.connect.clusters.os')
    def test_describe_cluster_output_yaml(self, mock_os):
        result = self.runner.invoke(connect,
                                    ['clusters', '--describe', '--cluster', self.cluster, '-n', self.namespace, '-o',
                                     'yaml'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkaconnects(self.cluster).namespace(self.namespace).output("yaml").build())

    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_delete_cluster(self, mock_os, mock_click_confirm):
        mock_click_confirm.return_value = True
        result = self.runner.invoke(connect, ['clusters', '--delete', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().delete().kafkaconnects(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.connect.clusters.click.confirm')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_delete_cluster_with_yes_flag(self, mock_os, mock_click_confirm):
        mock_click_confirm.return_value = False
        result = self.runner.invoke(connect,
                                    ['clusters', '--delete', '--cluster', self.cluster, '-n', self.namespace, '-y'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().delete().kafkaconnects(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.connect.clusters.os')
    def test_alter_cluster_without_parameters(self, mock_os):
        result = self.runner.invoke(connect, ['clusters', '--alter', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().edit().kafkaconnects(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_alter_cluster_with_replica_param(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-connect.yaml') as file:
            connect_yaml = file.read()

            mock_get_resource_yaml.return_value = connect_yaml

            result = self.runner.invoke(connect,
                                        ['clusters', '--alter', '--cluster', self.cluster, '--replicas', 3, '-n',
                                         self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-connect_with_three_replicas.yaml') as file:
                expected_connect_yaml = file.read()
                result_connect_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connect_yaml == result_connect_yaml

    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_alter_cluster_with_different_config_file(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-connect.yaml') as file:
            connect_yaml = file.read()

            mock_get_resource_yaml.return_value = connect_yaml

            result = self.runner.invoke(connect, ['clusters', '--alter', '--cluster', self.cluster,
                                                  'files/connect_with_zip_jar_plugins.properties', '-n',
                                                  self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-connect_with_zip_jar_plugins.yaml') as file:
                expected_connect_yaml = file.read()
                result_connect_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connect_yaml == result_connect_yaml

    @mock.patch('kfk.commands.connect.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.connect.clusters.os')
    def test_alter_cluster_with_only_image_config_file(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-connect.yaml') as file:
            connect_yaml = file.read()

            mock_get_resource_yaml.return_value = connect_yaml

            result = self.runner.invoke(connect, ['clusters', '--alter', '--cluster', self.cluster,
                                                  'files/connect_with_only_image.properties', '-n',
                                                  self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-connect_with_image.yaml') as file:
                expected_connect_yaml = file.read()
                result_connect_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connect_yaml == result_connect_yaml



from unittest import TestCase, mock
from click.testing import CliRunner
from kfk.commands.topics import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkTopics(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"

    def test_no_option(self):
        result = self.runner.invoke(kfk, ['topics', '--topic', self.topic, '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        assert "Missing options: kfk topics" in result.output

    @mock.patch('kfk.commands.topics.os')
    def test_list_topics(self, mock_os):
        result = self.runner.invoke(kfk, ['topics', '--list', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkatopics().label("strimzi.io/cluster={cluster}").namespace(
                self.namespace).build().format(
                cluster=self.cluster))

    @mock.patch('kfk.commands.topics.os')
    def test_describe_topic(self, mock_os):
        result = self.runner.invoke(kfk, ['topics', '--describe', '--topic', self.topic, '-c', self.cluster, '-n',
                                          self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkatopics(self.topic).namespace(self.namespace).build())

    @mock.patch('kfk.commands.topics.os')
    def test_describe_topic_output_yaml(self, mock_os):
        result = self.runner.invoke(kfk, ['topics', '--describe', '--topic', self.topic, '-c', self.cluster, '-n',
                                          self.namespace, '-o', 'yaml'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkatopics(self.topic).namespace(self.namespace).output("yaml").build())

    @mock.patch('kfk.commands.topics.os')
    def test_describe_topic_native(self, mock_os):
        result = self.runner.invoke(kfk, ['topics', '--describe', '--topic', self.topic, '-c', self.cluster, '-n',
                                          self.namespace, '--native'])
        assert result.exit_code == 0
        native_command = "bin/kafka-topics.sh --bootstrap-server {cluster}-kafka-brokers:9092 --describe --topic {" \
                         "topic}"
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(topic=self.topic, cluster=self.cluster))

    @mock.patch('kfk.commons.transfer_file_to_container')
    @mock.patch('kfk.commands.topics.os')
    def test_describe_topic_native_with_command_config(self, mock_os, mock_transfer_file_to_container):
        result = self.runner.invoke(kfk, ['topics', '--describe', '--topic', self.topic, '--command-config',
                                          'files/client.properties', '-c', self.cluster, '-n', self.namespace,
                                          '--native'])
        assert result.exit_code == 0
        native_command = "bin/kafka-topics.sh --bootstrap-server {cluster}-kafka-brokers:9093 --describe --topic {" \
                         "topic} --command-config /tmp/client.properties;rm -rf /tmp/truststore.jks;rm -rf " \
                         "/tmp/user.p12;rm -rf /tmp/client.properties;"
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(topic=self.topic, cluster=self.cluster))

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commands.topics.os')
    def test_create_topic(self, mock_os, mock_create_temp_file):
        result = self.runner.invoke(kfk,
                                    ['topics', '--create', '--topic', self.topic, '--partitions', '12',
                                     '--replication-factor', '3', '-c',
                                     self.cluster, '-n',
                                     self.namespace])

        assert result.exit_code == 0

        with open(r'files/yaml/topic.yaml') as file:
            expected_topic_yaml = file.read()
            result_topic_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commands.topics.os')
    def test_create_topic_with_config(self, mock_os, mock_create_temp_file):
        result = self.runner.invoke(kfk,
                                    ['topics', '--create', '--topic', self.topic, '--partitions', '24',
                                     '--replication-factor', '3', '--config', 'min.insync.replicas=3', '-c',
                                     self.cluster, '-n', self.namespace])

        assert result.exit_code == 0

        with open(r'files/yaml/topic_with_one_config.yaml') as file:
            expected_topic_yaml = file.read()
            result_topic_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_topic_yaml == result_topic_yaml

    def test_create_topic_without_required_params(self):
        result = self.runner.invoke(kfk,
                                    ['topics', '--create', '--topic', self.topic, '-c',
                                     self.cluster, '-n',
                                     self.namespace])
        assert result.exit_code == 2

    @mock.patch('kfk.commands.topics.os')
    def test_delete_topic(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['topics', '--delete', '--topic', self.topic, '-c', self.cluster, '-n',
                                     self.namespace])

        assert result.exit_code == 0

        mock_os.system.assert_called_with(Kubectl().delete().kafkatopics(self.topic).namespace(self.namespace).build())

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_alter_topic_with_no_params(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/topic.yaml') as file:
            expected_topic_yaml = file.read()
            mock_get_resource_yaml.return_value = expected_topic_yaml

            result = self.runner.invoke(kfk,
                                        ['topics', '--alter', '--topic', self.topic, '-c', self.cluster, '-n',
                                         self.namespace])

            assert result.exit_code == 0

            result_topic_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_alter_topic_without_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/topic.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['topics', '--alter', '--topic', self.topic, '--partitions', '24',
                                         '--replication-factor', '3', '-c', self.cluster, '-n', self.namespace])

            assert result.exit_code == 0

            with open(r'files/yaml/topic_without_config.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_alter_topic_with_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/topic.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['topics', '--alter', '--topic', self.topic, '--partitions', '24',
                                         '--replication-factor', '3', '--config', 'min.insync.replicas=3', '-c',
                                         self.cluster, '-n', self.namespace])

            assert result.exit_code == 0

            with open(r'files/yaml/topic_with_one_config.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_alter_topic_with_two_configs(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/topic.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['topics', '--alter', '--topic', self.topic, '--partitions', '24',
                                         '--replication-factor', '3', '--config', 'min.insync.replicas=3',
                                         '--config', 'cleanup.policy=compact', '-c', self.cluster, '-n',
                                         self.namespace])

            assert result.exit_code == 0

            with open(r'files/yaml/topic_with_two_configs.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_alter_topic_with_two_configs_delete_one_config(self, mock_os, mock_get_resource_yaml,
                                                            mock_create_temp_file):
        with open(r'files/yaml/topic_with_two_configs.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['topics', '--alter', '--topic', self.topic, '--delete-config',
                                         'cleanup.policy', '-c', self.cluster, '-n', self.namespace])

            assert result.exit_code == 0

            with open(r'files/yaml/topic_with_one_config.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_alter_topic_with_two_configs_delete_two_configs(self, mock_os, mock_get_resource_yaml,
                                                             mock_create_temp_file):
        with open(r'files/yaml/topic_with_two_configs.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['topics', '--alter', '--topic', self.topic, '--delete-config',
                                         'cleanup.policy', '--delete-config',
                                         'min.insync.replicas', '-c', self.cluster, '-n', self.namespace])

            assert result.exit_code == 0

            with open(r'files/yaml/topic_without_config.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml



from unittest import TestCase, mock
from click.testing import CliRunner
from kfk.commands.acls import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkAcls(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.kafka_cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"
        self.group = "my-group"
        self.user = "my-user"

    @mock.patch('kfk.commands.acls.os')
    def test_list_all_acls(self, mock_os):
        result = self.runner.invoke(kfk, ['acls', '--list', '-n', self.namespace, '-c', self.kafka_cluster])
        assert result.exit_code == 0
        native_command = "bin/kafka-acls.sh --authorizer-properties zookeeper.connect=localhost:12181 --list  "
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{kafka_cluster}-zookeeper-0").container("zookeeper").namespace(
                self.namespace).exec_command(native_command).build().format(kafka_cluster=self.kafka_cluster))

    @mock.patch('kfk.commands.acls.os')
    def test_list_topic_acls(self, mock_os):
        result = self.runner.invoke(kfk, ['acls', '--list', '--topic', self.topic, '-n', self.namespace, '-c',
                                          self.kafka_cluster])
        assert result.exit_code == 0
        native_command = "bin/kafka-acls.sh --authorizer-properties zookeeper.connect=localhost:12181 --list --topic " \
                         "{topic} ".format(topic=self.topic)
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{kafka_cluster}-zookeeper-0").container("zookeeper").namespace(
                self.namespace).exec_command(native_command).build().format(kafka_cluster=self.kafka_cluster))

    @mock.patch('kfk.commands.acls.os')
    def test_list_group_acls(self, mock_os):
        result = self.runner.invoke(kfk, ['acls', '--list', '--group', self.group, '-n', self.namespace, '-c',
                                          self.kafka_cluster])
        assert result.exit_code == 0
        native_command = "bin/kafka-acls.sh --authorizer-properties zookeeper.connect=localhost:12181 --list  --group" \
                         " {group}".format(group=self.group)
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{kafka_cluster}-zookeeper-0").container("zookeeper").namespace(
                self.namespace).exec_command(native_command).build().format(kafka_cluster=self.kafka_cluster))

    @mock.patch('kfk.commands.acls.os')
    def test_add_acls_without_principal(self, mock_os):
        result = self.runner.invoke(kfk, ['acls', '--add', '--topic', self.topic, '-n', self.namespace, '-c',
                                          self.kafka_cluster])
        assert result.exit_code == 2

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_add_topic_acl(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['acls', '--add', '--topic', self.topic, '--allow-principal',
                                         'User:' + self.user, '--operation', 'Read', '-n', self.namespace, '-c',
                                         self.kafka_cluster])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_remove_topic_acl(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['acls', '--remove', '--topic', self.topic, '--allow-principal',
                                         'User:' + self.user, '--operation', 'Read', '-n', self.namespace, '-c',
                                         self.kafka_cluster])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authentication_tls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_add_topic_acl_to_one_acl(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['acls', '--add', '--topic', self.topic, '--allow-principal',
                                         'User:' + self.user, '--operation', 'Write', '-n', self.namespace, '-c',
                                         self.kafka_cluster])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_two_topic_acls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_remove_topic_acl_from_two_acls(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_two_topic_acls.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['acls', '--remove', '--topic', self.topic, '--allow-principal',
                                         'User:' + self.user, '--operation', 'Write', '-n', self.namespace, '-c',
                                         self.kafka_cluster])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_add_topic_acl_with_two_operations(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['acls', '--add', '--topic', self.topic, '--allow-principal',
                                         'User:' + self.user, '--operation', 'Write', '--operation', 'Describe', '-n',
                                         self.namespace, '-c', self.kafka_cluster])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_three_topic_acls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_add_group_acl(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authorization_with_one_topic_acl.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['acls', '--add', '--group', self.group, '--allow-principal',
                                         'User:' + self.user, '--operation', 'Read', '-n', self.namespace, '-c',
                                         self.kafka_cluster])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_authorization_with_one_topic_and_one_group_acls.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml



from unittest import TestCase, mock

import click
import yaml

from click.testing import CliRunner
from kfk.commands.connect.connectors import connect
from kfk.kubectl_command_builder import Kubectl


class TestKfkConnectors(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-connect-cluster"
        self.namespace = "kafka"
        self.connector_config_file = "files/twitter-connector.properties"
        self.connector = "twitter-source-connector"

    def test_no_option(self):
        result = self.runner.invoke(connect,
                                    ['connectors', self.connector_config_file, '-c', self.cluster, '-n',
                                     self.namespace])
        assert result.exit_code == 0
        assert "Missing options: kfk connectors" in result.output

    @mock.patch('kfk.commands.connect.connectors.os')
    def test_list_connectors(self, mock_os):
        result = self.runner.invoke(connect, ['connectors', '--list', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkaconnectors().label("strimzi.io/cluster={cluster}").namespace(
                self.namespace).build().format(
                cluster=self.cluster))

    @mock.patch('kfk.commands.connect.connectors.os')
    def test_describe_connector(self, mock_os):
        result = self.runner.invoke(connect,
                                    ['connectors', '--describe', '--connector', self.connector, '-c', self.cluster,
                                     '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkaconnectors(self.connector).namespace(self.namespace).build())

    @mock.patch('kfk.commands.connect.connectors.os')
    def test_describe_connector_output_yaml(self, mock_os):
        result = self.runner.invoke(connect,
                                    ['connectors', '--describe', '--connector', self.connector, '-c', self.cluster,
                                     '-n', self.namespace, '-o', 'yaml'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkaconnectors(self.connector).namespace(self.namespace).output("yaml").build())

    @mock.patch('kfk.commands.connect.connectors.os')
    def test_delete_connector(self, mock_os):
        result = self.runner.invoke(connect,
                                    ['connectors', '--delete', '--connector', self.connector, '-c', self.cluster, '-n',
                                     self.namespace])

        assert result.exit_code == 0

        mock_os.system.assert_called_with(
            Kubectl().delete().kafkaconnectors(self.connector).namespace(self.namespace).build())

    def test_create_connector_without_config_file(self):
        result = self.runner.invoke(connect,
                                    ['connectors', '--create', '-c', self.cluster, '-n', self.namespace])

        assert result.exit_code == 2

    @mock.patch('kfk.commands.connect.connectors.create_temp_file')
    @mock.patch('kfk.commands.connect.connectors.os')
    def test_create_connector(self, mock_os, mock_create_temp_file):
        result = self.runner.invoke(connect,
                                    ['connectors', '--create', self.connector_config_file, '-c',
                                     self.cluster, '-n', self.namespace])

        assert result.exit_code == 0

        with open(r'files/yaml/kafka-connect-connector-twitter.yaml') as file:
            expected_connector_yaml = file.read()
            result_connector_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_connector_yaml == result_connector_yaml

    def test_alter_connector_without_config_file(self):
        result = self.runner.invoke(connect,
                                    ['connectors', '--alter', '-c', self.cluster, '-n', self.namespace])

        assert result.exit_code == 2

    @mock.patch('kfk.commands.connect.connectors.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.connect.connectors.os')
    def test_alter_connector(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-connect-connector-twitter.yaml') as file:
            connector_yaml = file.read()
            mock_get_resource_yaml.return_value = connector_yaml

            result = self.runner.invoke(connect,
                                        ['connectors', '--alter',
                                         'files/twitter_connector_with_config_change.properties', '-c',
                                         self.cluster, '-n', self.namespace])

            assert result.exit_code == 0

            with open(r'files/yaml/kafka-connect-connector-twitter_with_config_change.yaml') as file:
                expected_connector_yaml = file.read()
                result_connector_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_connector_yaml == result_connector_yaml



from unittest import TestCase, mock
from click.testing import CliRunner
from kfk.commands.configs import kfk
from kfk.kubectl_command_builder import Kubectl
from kfk.constants import *
from kfk.messages import *


class TestKfkConfigs(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.topic = "my-topic"
        self.user = "my-user"
        self.broker_count = 3

    def test_no_option(self):
        result = self.runner.invoke(kfk, ['configs', '--entity-type', 'topics', '--entity-name', self.topic, '-c',
                                          self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        assert "Missing options: kfk configs" in result.output

    def test_wrong_entity_type(self):
        result = self.runner.invoke(kfk, ['configs', '--entity-type', 'foos', '--entity-name', self.topic, '-c',
                                          self.cluster, '-n', self.namespace])
        assert result.exit_code == 2

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_add_one_topic_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/topic_without_config.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--add-config', 'min.insync.replicas=3', '--entity-type',
                                         'topics', '--entity-name', self.topic, '-c', self.cluster, '-n',
                                         self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/topic_with_one_config.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_add_two_topic_configs(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/topic_without_config.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--add-config',
                                         'min.insync.replicas=3,cleanup.policy=compact', '--entity-type', 'topics',
                                         '--entity-name', self.topic, '-c', self.cluster,
                                         '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/topic_with_two_configs.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.topics.os')
    def test_delete_one_topic_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/topic_with_two_configs.yaml') as file:
            topic_yaml = file.read()
            mock_get_resource_yaml.return_value = topic_yaml

            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--delete-config', 'cleanup.policy', '--entity-type',
                                         'topics', '--entity-name', self.topic, '-c', self.cluster, '-n',
                                         self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/topic_with_one_config.yaml') as file:
                expected_topic_yaml = file.read()
                result_topic_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_topic_yaml == result_topic_yaml

    @mock.patch('kfk.commands.topics.os')
    def test_describe_topic_config(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['configs', '--describe', '--entity-type', 'topics',
                                     '--entity-name', self.topic, '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkatopics(self.topic).namespace(self.namespace).build())

    @mock.patch('kfk.commands.configs.os')
    def test_describe_topic_config_native(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['configs', '--describe', '--entity-type', 'topics',
                                     '--entity-name', self.topic, '--native', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0

        native_command = "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092 --entity-type " \
                         "topics --describe --entity-name {entity_name}"
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(cluster=self.cluster, entity_name=self.topic))

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_add_one_user_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_scram.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--add-config', 'request_percentage=55', '--entity-type',
                                         'users', '--entity-name', self.user, '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_one_quota.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_add_two_user_configs(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_authentication_scram.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--add-config',
                                         'request_percentage=55,consumer_byte_rate=2097152', '--entity-type', 'users',
                                         '--entity-name', self.user, '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_two_quotas.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.users.os')
    def test_delete_one_user_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/user_with_two_quotas.yaml') as file:
            user_yaml = file.read()
            mock_get_resource_yaml.return_value = user_yaml

            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--delete-config', 'consumer_byte_rate', '--entity-type',
                                         'users', '--entity-name', self.user, '-c', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/user_with_one_quota.yaml') as file:
                expected_user_yaml = file.read()
                result_user_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_user_yaml == result_user_yaml

    @mock.patch('kfk.commands.users.os')
    def test_describe_user_config(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['configs', '--describe', '--entity-type', 'users',
                                     '--entity-name', self.user, '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkausers(self.user).namespace(self.namespace).build())

    @mock.patch('kfk.commands.configs.os')
    def test_describe_user_config_native(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['configs', '--describe', '--entity-type', 'users',
                                     '--entity-name', self.user, '--native', '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0

        native_command = "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092 --entity-type " \
                         "users --describe --entity-name CN={entity_name}"
        mock_os.system.assert_called_with(
            Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                native_command).build().format(cluster=self.cluster, entity_name=self.user))

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_add_one_broker_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral.yaml') as file:
            cluster_yaml = file.read()
            mock_get_resource_yaml.return_value = cluster_yaml
            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--add-config', 'unclean.leader.election.enable=true',
                                         '--entity-type', 'brokers', '--entity-name', self.cluster, '-c', self.cluster,
                                         '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_with_one_additional_config.yaml') as file:
                expected_cluster_yaml = file.read()
                result_cluster_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_cluster_yaml == result_cluster_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_delete_one_broker_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral_with_one_additional_config.yaml') as file:
            cluster_yaml = file.read()
            mock_get_resource_yaml.return_value = cluster_yaml
            result = self.runner.invoke(kfk,
                                        ['configs', '--alter', '--delete-config', 'unclean.leader.election.enable',
                                         '--entity-type', 'brokers', '--entity-name', 'all', '-c', self.cluster,
                                         '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_two_additional_configs_deleted.yaml') as file:
                expected_cluster_yaml = file.read()
                result_cluster_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_cluster_yaml == result_cluster_yaml

    @mock.patch('kfk.commands.clusters.os')
    def test_describe_broker_config(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['configs', '--describe', '--entity-type', 'brokers',
                                     '--entity-name', "all", '-c', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().describe().kafkas(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.configs.os')
    def test_describe_broker_config_native(self, mock_os, mock_get_config_resource_yaml):
        with open(r'files/yaml/kafka-config.yaml') as file:
            config_yaml = file.read()
            mock_get_config_resource_yaml.return_value = config_yaml
            result = self.runner.invoke(kfk,
                                        ['configs', '--describe', '--entity-type', 'brokers', '--native', '-c',
                                         self.cluster, '-n', self.namespace])
            assert result.exit_code == 0
            assert "All user provided configs for brokers in the cluster are:" in result.output
            assert "log.message.format.version=2.6" in result.output
            assert "offsets.topic.replication.factor=3" in result.output
            assert "transaction.state.log.min.isr=2" in result.output
            assert "transaction.state.log.replication.factor=3" in result.output

            native_command = "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092 --entity-type " \
                             "brokers --describe"

            mock_os.system.assert_called_with(
                Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                    native_command).build().format(cluster=self.cluster, entity_name=self.broker_count - 1))

    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.configs.os')
    def test_describe_broker_config_native_with_id(self, mock_os, mock_get_config_resource_yaml):
        with open(r'files/yaml/kafka-config.yaml') as file:
            config_yaml = file.read()
            mock_get_config_resource_yaml.return_value = config_yaml
            result = self.runner.invoke(kfk,
                                        ['configs', '--describe', '--entity-type', 'brokers',
                                         '--entity-name', '0', '--native', '-c', self.cluster, '-n',
                                         self.namespace])
            assert result.exit_code == 0
            assert "All user provided configs for brokers in the cluster are:" in result.output
            assert "log.message.format.version=2.6" in result.output
            assert "offsets.topic.replication.factor=3" in result.output
            assert "transaction.state.log.min.isr=2" in result.output
            assert "transaction.state.log.replication.factor=3" in result.output

            native_command = "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:9092 --entity-type " \
                             "brokers --describe --entity-name 0"

            mock_os.system.assert_called_with(
                Kubectl().exec("-it", "{cluster}-kafka-0").container("kafka").namespace(self.namespace).exec_command(
                    native_command).build().format(cluster=self.cluster, entity_name=self.broker_count - 1))



from unittest import TestCase, mock
from click.testing import CliRunner
from kfk.commands.clusters import kfk
from kfk.kubectl_command_builder import Kubectl


class TestKfkClusters(TestCase):

    def setUp(self):
        self.runner = CliRunner()
        self.cluster = "my-cluster"
        self.namespace = "kafka"
        self.new_cluster_name = "my-cluster-with-new-name"

    def test_no_option(self):
        result = self.runner.invoke(kfk, ['clusters', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        assert "Missing options: kfk clusters" in result.output

    @mock.patch('kfk.commands.clusters.os')
    def test_list_clusters(self, mock_os):
        result = self.runner.invoke(kfk, ['clusters', '--list', '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().get().kafkas().namespace(self.namespace).build())

    @mock.patch('kfk.commands.clusters.os')
    def test_list_clusters_all_namespaces(self, mock_os):
        result = self.runner.invoke(kfk, ['clusters', '--list'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().get().kafkas().namespace().build())

    @mock.patch('kfk.commands.clusters.os')
    def test_describe_cluster(self, mock_os):
        result = self.runner.invoke(kfk, ['clusters', '--describe', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().describe().kafkas(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.clusters.os')
    def test_describe_cluster_output_yaml(self, mock_os):
        result = self.runner.invoke(kfk,
                                    ['clusters', '--describe', '--cluster', self.cluster, '-n', self.namespace, '-o',
                                     'yaml'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(
            Kubectl().get().kafkas(self.cluster).namespace(self.namespace).output("yaml").build())

    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_without_parameters(self, mock_os):
        result = self.runner.invoke(kfk, ['clusters', '--alter', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().edit().kafkas(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_with_one_additional_config(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral.yaml') as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(kfk, ['clusters', '--alter', '--config', 'unclean.leader.election.enable=true',
                                              '--cluster', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_with_one_additional_config.yaml') as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_with_two_additional_configs(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral.yaml') as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(kfk, ['clusters', '--alter', '--config', 'unclean.leader.election.enable=true',
                                              '--config', 'log.retention.hours=168',
                                              '--cluster', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_with_two_additional_configs.yaml') as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_with_two_additional_configs_delete_one_config(self, mock_os, mock_get_resource_yaml,
                                                                         mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral_with_two_additional_configs.yaml') as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(kfk, ['clusters', '--alter', '--delete-config', 'log.retention.hours',
                                              '--cluster', self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_with_one_additional_config.yaml') as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_with_two_additional_configs_delete_two_configs(self, mock_os, mock_get_resource_yaml,
                                                                          mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral_with_two_additional_configs.yaml') as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(kfk, ['clusters', '--alter', '--delete-config', 'log.retention.hours',
                                              '--delete-config', 'unclean.leader.election.enable', '--cluster',
                                              self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_two_additional_configs_deleted.yaml') as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_with_one_replica(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral_name_updated.yaml') as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(kfk, ['clusters', '--alter', '--replicas', 1, '--cluster', self.cluster, '-n',
                                              self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_with_one_replica.yaml') as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_with_two_replicas(self, mock_os, mock_get_resource_yaml, mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral_name_updated.yaml') as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(kfk, ['clusters', '--alter', '--replicas', 2, '--cluster', self.cluster, '-n',
                                              self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_with_two_replicas.yaml') as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commons.get_resource_yaml')
    @mock.patch('kfk.commands.clusters.os')
    def test_alter_cluster_with_one_replica_one_zk_replica(self, mock_os, mock_get_resource_yaml,
                                                           mock_create_temp_file):
        with open(r'files/yaml/kafka-ephemeral_name_updated.yaml') as file:
            kafka_yaml = file.read()
            mock_get_resource_yaml.return_value = kafka_yaml
            result = self.runner.invoke(kfk, ['clusters', '--alter', '--replicas', 1, '--zk-replicas', 1, '--cluster',
                                              self.cluster, '-n', self.namespace])
            assert result.exit_code == 0

            with open(r'files/yaml/kafka-ephemeral_with_one_replica_one_zk_replica.yaml') as file:
                expected_kafka_yaml = file.read()
                result_kafka_yaml = mock_create_temp_file.call_args[0][0]
                assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_delete_cluster(self, mock_os, mock_click_confirm):
        mock_click_confirm.return_value = True
        result = self.runner.invoke(kfk, ['clusters', '--delete', '--cluster', self.cluster, '-n', self.namespace])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().delete().kafkas(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_delete_cluster_with_yes_flag(self, mock_os, mock_click_confirm):
        mock_click_confirm.return_value = False
        result = self.runner.invoke(kfk,
                                    ['clusters', '--delete', '--cluster', self.cluster, '-n', self.namespace, '-y'])
        assert result.exit_code == 0
        mock_os.system.assert_called_with(Kubectl().delete().kafkas(self.cluster).namespace(self.namespace).build())

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commands.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_create_cluster(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor, mock_create_temp_file):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(kfk,
                                    ['clusters', '--create', '--cluster', self.new_cluster_name, '-n', self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-ephemeral_name_updated.yaml') as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commands.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_create_cluster_with_yes_flag(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                          mock_create_temp_file):
        mock_click_confirm.return_value = False

        result = self.runner.invoke(kfk,
                                    ['clusters', '--create', '--cluster', self.new_cluster_name, '-n', self.namespace,
                                     '-y'])
        assert result.exit_code == 0

        mock_open_file_in_system_editor.assert_not_called()
        with open(r'files/yaml/kafka-ephemeral_name_updated.yaml') as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commands.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_create_cluster_with_one_replica(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                             mock_create_temp_file):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(kfk,
                                    ['clusters', '--create', '--cluster', self.new_cluster_name, '--replicas', 1, '-n',
                                     self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-ephemeral_with_one_replica.yaml') as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commands.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_create_cluster_with_two_replicas(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                              mock_create_temp_file):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(kfk,
                                    ['clusters', '--create', '--cluster', self.new_cluster_name, '--replicas', 2, '-n',
                                     self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-ephemeral_with_two_replicas.yaml') as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commands.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_create_cluster_with_three_replicas(self, mock_os, mock_click_confirm, mock_open_file_in_system_editor,
                                                mock_create_temp_file):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(kfk,
                                    ['clusters', '--create', '--cluster', self.new_cluster_name, '--replicas', 3, '-n',
                                     self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-ephemeral_name_updated.yaml') as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml

    @mock.patch('kfk.commands.clusters.create_temp_file')
    @mock.patch('kfk.commands.clusters.open_file_in_system_editor')
    @mock.patch('kfk.commands.clusters.click.confirm')
    @mock.patch('kfk.commands.clusters.os')
    def test_create_cluster_with_one_replica_one_zk_replica(self, mock_os, mock_click_confirm,
                                                            mock_open_file_in_system_editor,
                                                            mock_create_temp_file):
        mock_click_confirm.return_value = True

        result = self.runner.invoke(kfk,
                                    ['clusters', '--create', '--cluster', self.new_cluster_name, '--replicas', 1,
                                     '--zk-replicas', 1, '-n', self.namespace])
        assert result.exit_code == 0

        with open(r'files/yaml/kafka-ephemeral_with_one_replica_one_zk_replica.yaml') as file:
            expected_kafka_yaml = file.read()
            result_kafka_yaml = mock_create_temp_file.call_args[0][0]
            assert expected_kafka_yaml == result_kafka_yaml
